#
# spec file for package rustup
#
# Copyright (c) 2023 SUSE LLC
# Copyright (c) 2021 William Brown <william@blackhats.net.au>
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define rust_version 1.56
%define obsolete_rust_versioned() \
Obsoletes:      %{1}1.55%{?2:-%{2}} < %{rust_version} \
Provides:       %{1}1.55%{?2:-%{2}} = %{rust_version} \
Obsoletes:      %{1}1.54%{?2:-%{2}} < %{rust_version} \
Provides:       %{1}1.54%{?2:-%{2}} = %{rust_version} \
Obsoletes:      %{1}1.53%{?2:-%{2}} < %{rust_version} \
Provides:       %{1}1.53%{?2:-%{2}} = %{rust_version} \
Obsoletes:      %{1}1.52%{?2:-%{2}} < %{rust_version} \
Obsoletes:      %{1}1.51%{?2:-%{2}} < %{rust_version}

Name:           rustup
Version:        1.26.0~0
Release:        150400.3.7.1
Summary:        A tool for managing user Rust toolchains
License:        (0BSD OR MIT OR Apache-2.0) AND (Apache-2.0 OR BSL-1.0) AND (Apache-2.0 OR ISC OR MIT) AND (Apache-2.0 OR MIT) AND (Apache-2.0 OR MIT OR Zlib) AND (Apache-2.0 OR MIT OR Zlib) AND (MIT OR Unlicense) AND (Apache-2.0 OR Zlib OR MIT) AND Apache-2.0 AND BSD-3-Clause AND CC0-1.0 AND ISC AND MIT
Group:          Development/Languages/Rust
URL:            https://github.com/rust-lang/rustup
Source0:        %{name}-%{version}.tar.zst
Source1:        vendor.tar.zst
Source2:        cargo_config
Source3:        %{name}-rpmlintrc
BuildRequires:  cargo-packaging
BuildRequires:  pkgconfig(openssl)
# For system linker
Requires:       gcc
ExclusiveArch:  %{rust_tier1_arches}

# Rustup now provides rust-analyzer which we take over.
Provides:       rust-analyzer = 2022.09.11
Obsoletes:      rust-analyzer < 2022.09.11

# Ensure we can upgrate from ancient versions of rust that used
# to ship these tools. We can probably remove this in the future.
%obsolete_rust_versioned rls
%obsolete_rust_versioned rust doc
%obsolete_rust_versioned rust src
%obsolete_rust_versioned rust analysis
%obsolete_rust_versioned cargo doc

Obsoletes:      cargo-doc < %{rust_version}
Obsoletes:      cargo-fmt < %{rust_version}
Obsoletes:      clippy < %{rust_version}
Obsoletes:      rls < %{rust_version}
Obsoletes:      rust-analysis < %{rust_version}
Obsoletes:      rust-doc < %{rust_version}
Obsoletes:      rust-src < %{rust_version}
Obsoletes:      rustfmt < %{rust_version}

Conflicts:      rust+cargo
Conflicts:      rust+rustc

%description
A tool to manager user Rust toolchains. This is generally used by developers
managing multiple parallel toolchains in their environment.

%prep
%setup -q
%setup -qa1
cp %{SOURCE2} .cargo/config
# Remove exec bits to prevent an issue in fedora shebang checking. Uncomment only if required.
find vendor -type f -name \*.rs -exec chmod -x '{}' \;

%build
%{cargo_build} --features=no-self-update

%install
# manual process
install -D -d -m 0755 %{buildroot}%{_bindir}
install -m 0755 %{_builddir}/%{name}-%{version}/target/release/rustup-init %{buildroot}%{_bindir}/rustup
# rustup-init and rustup are the same binary, but that binary behaves
# differently based on the name it's executed by.
# rustup-init takes care of the initial setup, which includes setting PATH,
# modifying .bashrc etc. It's supposed to be called only once.
# rustup doesn't perform those steps and it only manages toolchains. It
# can be used the entire time.
#
# https://github.com/rust-lang/rustup/blob/master/src/lib.rs#L21
ln -sf rustup %{buildroot}%{_bindir}/rustc
ln -sf rustup %{buildroot}%{_bindir}/rustdoc
ln -sf rustup %{buildroot}%{_bindir}/cargo
ln -sf rustup %{buildroot}%{_bindir}/rust-analyzer
ln -sf rustup %{buildroot}%{_bindir}/rust-lldb
ln -sf rustup %{buildroot}%{_bindir}/rust-gdb
ln -sf rustup %{buildroot}%{_bindir}/rust-gdbgui
ln -sf rustup %{buildroot}%{_bindir}/rls
ln -sf rustup %{buildroot}%{_bindir}/cargo-clippy
ln -sf rustup %{buildroot}%{_bindir}/clippy-driver
ln -sf rustup %{buildroot}%{_bindir}/cargo-miri
ln -sf rustup %{buildroot}%{_bindir}/rustfmt
ln -sf rustup %{buildroot}%{_bindir}/cargo-fmt

# shell completions
install -D -d -m 0755 %{buildroot}%{_datadir}/bash-completion/completions
ls %{buildroot}%{_bindir}
ls %{buildroot}%{_bindir}/rustup

%{buildroot}%{_bindir}/rustup completions bash > %{buildroot}%{_datadir}/bash-completion/completions/rustup
%{buildroot}%{_bindir}/rustup completions bash cargo > %{buildroot}%{_datadir}/bash-completion/completions/cargo
install -D -d -m 0755 %{buildroot}%{_datadir}/zsh/site-functions
%{buildroot}%{_bindir}/rustup completions zsh > %{buildroot}%{_datadir}/zsh/site-functions/_rustup
%{buildroot}%{_bindir}/rustup completions zsh cargo > %{buildroot}%{_datadir}/zsh/site-functions/_cargo

%files
%{_bindir}/rustup
# %{_bindir}/rustup-init
%{_bindir}/rustc
%{_bindir}/rustdoc
%{_bindir}/cargo
%{_bindir}/rust-analyzer
%{_bindir}/rust-lldb
%{_bindir}/rust-gdb
%{_bindir}/rust-gdbgui
%{_bindir}/rls
%{_bindir}/cargo-clippy
%{_bindir}/clippy-driver
%{_bindir}/cargo-miri
%{_bindir}/rustfmt
%{_bindir}/cargo-fmt
%{_datadir}/bash-completion/completions/cargo
%{_datadir}/bash-completion/completions/rustup
%dir %{_datadir}/zsh
%dir %{_datadir}/zsh/site-functions
%{_datadir}/zsh/site-functions/_cargo
%{_datadir}/zsh/site-functions/_rustup

%changelog
* Sat Jun  3 2023 william.brown@suse.com
- jsc#SLE-18626 - update rustup to support newer toolchains and language features
- Update to version 1.26.0~0:
  * Update commit sha in rustup-init.sh again
  * Update changelog to add more contributors
  * Remove test
  * Better error instead of show the help
  * Print help when a ValueValidation error occurs
  * Revert `Update Rust crate term to v0.7.0`
  * Update network-proxies.md
  * Fix RUSTUP_PERMIT_COPY_RENAME condition so it is actually used
  * Add a test for only options command
  * Update UI tests
  * Set SubcommandRequiredElseHelp correctly
  * Add preload directive to HSTS header
  * Remove useless link
  * Update changelog about remove GPG signature support
  * Update toolchain for android
  * Use llvm-ranlib
  * Use llvm-ar
  * Update toolchain
  * Revert "Temporarily disable the android tests"
  * Mention remove GPG signature support
  * Add all contributors
  * Update CHANGELOG  for v1.26.0
  * Update commit sha in rustup-init.sh again
  * Remove `show keys` subcommand
  * Move trycmd to dev-dependencies
  * Remove GPG signature support
  * Clippy fixes
  * Clippy fixes
  * Add RUSTUP_WINDOWS_PATH_ADD_BIN
  * Don't add toolchain bin to PATH on Windows
  * Fix broken CI for rust-x86_64-unknown-illumos
  * Regen the GitHub actions for windows
  * Add target env for windows
  * Update commit sha in rustup-init.sh again
  * Bump fs_at from v0.1.1 to v0.1.3
  * Update commit sha in rustup-init.sh
  * Bump version to 1.26.0
  * Remove extra new line after help
  * Make match statements better
  * Run the tests on x86_64-pc-windows-msvc
  * Update rustup mode tests
  * Update setup mode tests
  * Fix broken test path
  * Build debug and release in different jobs
  * Cap the maximum hard links per file
  * Update Rust crate trycmd to 0.14.13
  * Bump remove_dir_all
  * Bump clap to 3.0
  * Put self update test servers in test root
  * Hardlink rustup-init in the test suite
  * Update Rust crate tempfile to 3.4
  * Run integration tests as a single binary
  * Make gen-workflows.sh executable
  * Remove use of hardlinks to symlinks on macOS.
  * Update Rust crate curl to 0.4.44
  * Fix deprecated set-output warnings
  * Update Rust crate once_cell to 1.17.1
  * Use workspace inheritance
  * Enable Windows aarch64 builds on Stable
  * Migrate HostGoesMissing
  * Migrate cli-rustup to with_scenario
  * Introduce constant dist dirs for testing
  * expect_ok takes mutable Config
  * change_dir takes mutable Config
  * Lock file maintenance
  * Fix some more clippy-related issues.
  * Do all builds together in CI
  * Fix build on Windows
  * Update Rust crate trycmd to 0.14.12
  * Migrate callers to the new names
  * Move calls from fn(&config) to self.fn
  * Update Rust crate winreg to 0.11
  * Temporarily disable the android tests
  * add some explains for `--default-toolchain none`
  * Add ui test for the rustup-init.sh
  * fmt
  * Report duration of executed test commands
  * Make test process stdout/err clearer
  * Test the up aliases exist via help text
  * Update ubuntu Docker tag to v22
  * Update actions/upload-artifact action to v3
  * Update Rust crate winreg to 0.10
  * doc: add links for the aarch64-apple-darwin files
  * Use `Command::get_envs` in env_var
  * Update comment about CARGO_HOME on Windows
  * Update Rust crate zstd to 0.12
  * Update Rust crate url to 2.3
  * Fix windows tests
  * Add test for cli ui
  * Update Rust crate anyhow to 1.0.69
  * Use beta and give more explanation
  * Make clippy::needless-lifetimes happy
  * Make clippy::needless-borrow happy
  * Fix chrono deprecation warnings
  * clippy::uninlined_format_args: Apply clippy suggestions
  * Bump the centos image version
  * Remove temporary fix for the docker ci
  * fix(deps): update rust crate opener to 0.5.2
  * Rename from EventResult to DownloadCallback
  * Fix `type_complexity` error
  * allow to quickly edit a page directly on github
  * Fix `derive_partial_eq_without_eq` clippy error
  * Update MSVC installation image highlights
  * Spelling and grammar fixes
  * Add type parameter to prepend_path
  * Remove space from comment
  * Fix prepend_unique_path test on Windows
  * Fix duplicated PATH entries
  * chore(deps): lock file maintenance
  * Update actions/checkout action to v3
  * Expand on when VS Community isn't appropriate
  * Highlight relevant area of images
  * doc: Recommend tracking Cargo.lock with rust-toolchain
  * Update Rust crate term to v0.7.0
  * Update Rust crate effective-limits to 0.5.5
  * Test Renovate branches directly
  * Lock file maintenance
  * Update Rust crate sequoia-openpgp to 1.13
  * Update Rust crate rs_tracing to 1.1.0
  * Update Rust crate retry to 1.3.1
  * Update Rust crate num_cpus to 1.15
  * Update Rust crate enum-map to 2.4.2
  * Update GitHub Actions templates
  * Update actions/cache action to v3
  * Add -x and enable SC1090
  * Fix shell check warning
  * Renovate tweaks
  * Update Rust crate anyhow to 1.0.68
  * Tweak initial config
  * Put renovate.json in a better place
  * Add renovate.json
  * Improve a help message for `doc --rustdoc`
  * Fix typo
  * Use auto deref
  * Add automatic resume flag when retrying download with curl
  * Add "Proxy" to terminology page
  * Remove needless borrow
  * Update to home 0.5.4
  * Fix broken GitHub Actions
  * Fix typos
  * Update code to use `home::env as home` as suggested
  * Add details of each setting in the toolchain file
  * Update home usage to new env fn
  * Bump pulldown-cmark version to 0.9
  * Let Clap wrap help texts automatically
  * Apply suggestions from code review
  * Improve Windows MSVC install instructions
  * Add support for loongarch64
  * Add DisplayVersion for rustup to registry on Windows.
  * Add `up` as an alias of `update`
  * Move rust-analyzer to DUP_TOOLS
  * Add rust-analyzer proxy.
  * Unify the msg of the prompt when downloading self-update
  * Improve error message when there is an error caused by override file
  * (changelog): Fix links to tags
  * Improved warning message for System-Rust-override
  * Clippy lint
  * Remove snapcraft build and runtime support entirely
  * Use `if` instead of `$?`
  * Correctly propagate subshell failures in rustup-init
  * (toolchain): Toolchain names must not be empty
* Thu Apr 13 2023 william.brown@suse.com
- bsc#1210345 - CVE-2023-26964 - hyper,h2: stream stacking when H2 processing HTTP2 RST_STREAM frames
* Thu Feb 23 2023 william.brown@suse.com
- bsc#1208552 - CVE-2022-31394 - header list size may allow denial of service
- Update to version 1.25.2~git0.17db695:
  * properly fix mips64el
  * update compiler paths in docker images
  * temporary fix for the docker ci
  * Add -x and enable SC1090
  * Fix shell check warning
  * update version in install script
  * update version to 1.25.2
  * add changelog entry
  * allow sha1 signing keys without a cutoff date
* Mon Jun 13 2022 meissner@suse.com
- added correct provides to the obsoletes, to get correct provides
  obsoletes pairs. (bsc#1200499)
* Mon Nov  1 2021 william.brown@suse.com
- Add ownership of directories that are considered unowned in leap15.3
* Mon Oct 11 2021 william.brown@suse.com
- Apply patch:
  * Symlink rustup-init when in no self update mode
- Add requires on gcc so that system linker is present as a dependency
* Sat Oct  9 2021 mrostecki@suse.com
- Add symlink to rustup-init. rustup and rustup-init are the same
  binary, but that binary behaves diferently based on the name it's
  executed by.
  rustup-init takes care of the initial setup, which includes
  setting PATH, modifying .bashrc etc. It's supposed to be called
  only once.
  rustup doesn't perform those steps and it only manages
  toolchains. It can be used the entire time.
- Install bash and zsh completion.
* Thu Oct  7 2021 william.brown@suse.com
- Update rustup to be primary replacement to rls and other developer tools
* Thu Sep 30 2021 william.brown@suse.com
- Update to version 1.24.3~git0.ce5817a9:
  * rustup-init.sh: Update for 1.24.3
  * Cargo.{toml,lock}: Update for 1.24.3
  * CHANGELOG: Update for 1.24.3
  * tests: Test case for TargetTriple::can_run()
  * install: improve heuristic for deprecation warning
  * unpack_ram: More graceful handling of low unpack ram
  * diskio: Correctly account the release of the final chunk
  * diskio: Add buffer_used() to Executor and verify it in the test
  * Support self-update modes
* Thu Sep 30 2021 william.brown@suse.com
- Initial commit
